package MLocarna;
    
use strict;
use warnings;

BEGIN {
    use Exporter   ();
    our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);
    
    # set the version for version checking
    $VERSION     = 1.00;
    # if using RCS/CVS, this may be preferred
    $VERSION = sprintf "%d.%03d", q$Revision: 1.4 $ =~ /(\d+)/g;
    
    @ISA         = qw(Exporter);
    @EXPORT      = qw(
		      &parse_mfasta
		      &compute_alignment_from_seqs
		      &compute_alignment_from_dps
		      &compute_alignment_score
		      &parse_alignment
		      );
    %EXPORT_TAGS = ( );     # eg: TAG => [ qw!name1 name2! ],
    
    # your exported package globals go here,
    # as well as any optionally exported functions
    @EXPORT_OK   = qw();
}
our @EXPORT_OK;


sub parse_mfasta {
    my ($file) = @_;
    my %mfasta;
    
    local *PMF_IN;
    
    open(PMF_IN,$file) || die "Cannot read $file\n";

    my $line=<PMF_IN>;
    while(defined($line)) {
	if ($line=~/^>\s*(.+)/) {
	    my $name=$1;
	    chomp $name;
	    $name =~ s/[^a-zA-Z\d]/_/g;
	    $name = substr $name,0,16;
	    
	    if (exists $mfasta{$name}) {
		print STDERR "Duplicate name in mfasta: $name\n";
		exit -1;
	    }
	    while (defined($line=<PMF_IN>) && ($line !~ /^>/)) {
		chomp $line;
		$line =~ s/\s+//g;
		$mfasta{$name} .= $line;
	    }
	}
    }
    return %mfasta;
}


sub compute_alignment {
    my ($seqA,$seqB,$locarna_params) =  @_;
    
    local *CA_IN;
    
    my $tmpfile="/tmp/mlocarna.$$.clustal";

    system("locarna.pl $seqA $seqB $locarna_params --clustal=$tmpfile >/dev/null");
    
    open(CA_IN,"$tmpfile") || die "Cannot read $tmpfile\n";
    
    my @content=<CA_IN>;
   
    close CA_IN;
    
    unlink $tmpfile;
    
    return @content;
}

sub compute_alignment_from_dps {
    my ($seqA_dp,$seqB_dp,$locarna_params) =  @_;

    local *CA_IN;

    my $tmpfile="/tmp/mlocarna.$$.clustal";

    my $cmd="locarna $seqA_dp $seqB_dp $locarna_params --clustal=$tmpfile >/dev/null";
    system $cmd || die "Command $cmd failed."; 
    
    open(CA_IN,"$tmpfile") || die "compute_alignment: Cannot read temporary file $tmpfile\n";
    
    my @content=<CA_IN>;
   
    close CA_IN;
    
    unlink $tmpfile;
    
    return @content;
}


sub compute_alignment_score {
    my @content=compute_alignment @_;
    # print "@content\n";
    $content[0] =~ /Score: ([\d\.\-]+)/ || die "Cannot return score.\n";
    my $score=$1;

    return $score;
}


sub parse_alignment {
    my ($nameA,$nameB,$alignment_ref) = @_;

    my @alignment = @{ $alignment_ref };
    
    my $aliA="";
    my $aliB="";
    
    for my $line (@alignment) {
	if ($line =~ /^$nameA\s+(.+)/) {
	    $aliA.=$1;
	} elsif ($line =~ /^$nameB\s+(.+)/) {
	    $aliB.=$1;
	}
    }
    return ($aliA,$aliB);
}


sub parse_pp_file {
    my ($filename)=@_;
    local *PP_IN;

    open(PP_IN,$filename) || die "Can not read $filename\n";
    
    my %aln;

    my $line;

    while (($line = <PP_IN>) !~ /^\#/) {
	if ($line =~ /^(.+)\s+(.+)/) {
	    $aln{$1}.=$2;
	}
    }
    
    my %pairprobs;
    while (($line = <PP_IN>)) {
	if ($line =~ /(\S+)\s+(\S+)\s+(\S+)/) {
	    my $i=$1;
	    my $j=$2;
	    my $p=$3;
	    $pairprobs{($i,$j)}=$p;
	}
    }
    close PP_IN;
    
    return %aln, %pairprobs;
}




## ------------------------------------------------------------

END {}

1;
