#include <math.h>

#include "alignment.hh"
#include "basepairs.hh"

using namespace std;

void Alignment::write(ostream &out, int width) const {
    write_clustal(out, width);
}

void Alignment::write_clustal(ostream &out, int width, int score) const {
    
    cout<<endl;

    const char loc_blank='~';

    uint alisize=a_.size();
    
    vector<sequence_t> aliString;
    
    aliString.resize(4);

    sequence_t &aliStrA=aliString[0];
    sequence_t &aliSeqA=aliString[1];
    sequence_t &aliSeqB=aliString[2];
    sequence_t &aliStrB=aliString[3];

    aliStrA.init_buffer("");
    aliSeqA.init_buffer(seqA_);
    aliSeqB.init_buffer(seqB_);
    aliStrB.init_buffer("");
    
    
    int lastA=1; // bases consumed in sequence A
    int lastB=1; // ---------- "" ------------ B
    
    for (uint i=0; i<alisize; i++) {
	// out << "("<<a_[i]<<","<<b_[i]<<") "; out.flush();
	for (int j=lastA; j<a_[i]; j++) {
	    aliStrA += strA_[j];
	    aliSeqA += seqA_[j]; lastA++;
	    aliSeqB += loc_blank;
	    aliStrB += loc_blank;
	}
	for (int j=lastB; j<b_[i]; j++) {
	    aliStrA += loc_blank;
	    aliSeqA += loc_blank;
	    aliSeqB += seqB_[j]; lastB++;
	    aliStrB += strB_[j];
	}
	if ( a_[i]==-1 ) {
	    aliStrA += '-';
	    aliSeqA += '-';
	} else {
	    aliStrA += strA_[a_[i]]; lastA++;
	    aliSeqA += seqA_[a_[i]];
	}
	if ( b_[i]==-1 ) {
	    aliStrB += '-';
	    aliSeqB += '-';
	} else {
	    aliStrB += strB_[b_[i]]; lastB++;
	    aliSeqB += seqB_[b_[i]];
	}
    }
       
    for (uint j=lastA; j<=seqA_.length(); j++) {
	aliStrA += strA_[j];
	aliSeqA += seqA_[j]; lastA++;
	aliSeqB += loc_blank;
	aliStrB += loc_blank;
    }
    for (uint j=lastB; j<=seqB_.length(); j++) {
	aliStrA += loc_blank;
	aliSeqA += loc_blank;
	aliSeqB += seqB_[j]; lastB++;
	aliStrB += strB_[j];
    }
    
    if (score>INT_MIN)
	out << "CLUSTAL --- LocARNA - Local Alignment of RNA --- Score: "<<score<<endl<<endl;

    uint length=aliSeqA.length();
    uint k=1;
    while (k <= length) {
	for (uint line=0;line<4; line++) {
	    aliString[line].write( out, k, min(length,k+width-1) );
	}
	out<<endl;
	k+=width;
    }
}


class plusvector: public vector<int> {

public:
  plusvector& operator += (int x) {
    push_back(x);
    return *this;
  }
};


/*
  write pp output, which can be reread for progressive alignment!
*/

void Alignment::write_pp(ostream &out, Bps &bpsA, Bps &bpsB, int width) const { 
  // wir wollen für A und B jeweils eine Abbildung von Alignmentspalte auf Sequenzposition,
  
  uint alisize = a_.size();

  int lastA=1; // bases consumed in sequence A
  int lastB=1; // ---------- "" ------------ B

  plusvector aliA;
  plusvector aliB;

  for (uint i=0; i<alisize; i++) {
    // out << "("<<a_[i]<<","<<b_[i]<<") "; out.flush();
    for (int j=lastA; j<a_[i]; j++) {
      aliA += j; lastA++;
      aliB += -2;
    }
    for (int j=lastB; j<b_[i]; j++) {
      aliA += -2;
      aliB += j; lastB++;
    }
    if ( a_[i]==-1 ) {
      aliA += -1;
    } else {
      aliA += a_[i]; lastA++;
    }
    if ( b_[i]==-1 ) {
      aliB += -1;
    } else {
      aliB += b_[i]; lastB++;
    }
  }
  
  for (uint j=lastA; j<=seqA_.length(); j++) {
    aliA += j; lastA++;
    aliB += -2;
  }
  for (uint j=lastB; j<=seqB_.length(); j++) {
    aliA += -2;
    aliB += j; lastB++;
  }
  /*
  for (int i=0; i<aliA.size(); i++) {
    cerr <<i<<":"<< aliA[i]<<";"<<aliB[i]<<" ";
  }
  cerr <<endl;
  */

  write(out, width);
    
    out <<"#"<<endl;
    
    double p_minA = bpsA.prob_min();
    double p_minB = bpsB.prob_min();
    double p_expA = bpsA.prob_exp();
    double p_expB = bpsB.prob_exp();
    
    int lenA=seqA_.length();
    int lenB=seqB_.length();

    double p_minMean =
	exp(
	    (log(p_minA)*seqA_.get_rows()
	     + log(p_minB)*seqB_.get_rows())
	    / (seqA_.get_rows() + seqB_.get_rows())
	    );

    for (uint i=0; i<aliA.size(); i++) {
	for (uint j=i+1; j<aliB.size(); j++) {
	    double pA;
	    if (aliA[i]<0 || aliA[j]<0)
		pA = 0;
	    else
		pA = bpsA.get_arc_prob(aliA[i], aliA[j]);
	    
	    double pB;
	    if (aliB[i]<0 || aliB[j]<0)
		pB = 0;
	    else
		pB = bpsB.get_arc_prob(aliB[i], aliB[j]);
	    
	    // here we compute consensus pair probabilities
	    
	    double p=0;
	    
	    // add p_min and normalize
	    /* not doing this leads to extinction of structure information in larger multiple alignments */
	    pA = max(p_expA, pA);
	    pB = max(p_expB, pB);
	    
	    // weighted geometric mean
	    p = exp(
		(log(pA)*seqA_.get_rows() + log(pB)*seqB_.get_rows()) / 
		//--------------------------------------------------- 
		(seqA_.get_rows() + seqB_.get_rows())
		);
	    	    
	    // geometric mean
	    // p = pow( pA * pB, 0.5 );
	    
	    
	    // weighted arithmetic mean
            // p = (seqA_.get_rows()*pA + seqB_.get_rows()*pB)/(seqA_.get_rows()+seqB_.get_rows());
	    
	    if (p>p_minMean) {
		out << i+1 << " " << j+1 << " " << p <<endl;
		// cerr << p <<" <- "<<pA<<","<<pB<<" : "<<p_minA<<","<<p_minB<<";"<<p_minMean<<endl;
	    }
	}
    }
}
