#ifndef ALIGNMENT_HH
#define ALIGNMENT_HH

#include <iostream>
#include "sequence.hh"


/* ============================================================
   class Alignment

   Store a structure-annotated sequence alignment
   
   Methods:
   - append
   for constructing the alignment in the traceback phase
   - write
   for writing the alignment to stdout
   - add_basepairA and add_basepairB
   for adding basepairs in the structures of sequence A and B
============================================================ */

class Alignment {
    const sequence_t seqA_;
    const sequence_t seqB_;
    
    std::vector<int> a_;
    std::vector<int> b_;

    std::vector<char> strA_;
    std::vector<char> strB_;
    
public:
    Alignment(const sequence_t &seqA,const sequence_t &seqB)
	: seqA_(seqA),seqB_(seqB) {
	strA_.resize(seqA_.length()+1);
	strB_.resize(seqB_.length()+1);
	for (std::vector<char>::iterator it=strA_.begin(); it!=strA_.end(); ++it) *it='.';
	for (std::vector<char>::iterator it=strB_.begin(); it!=strB_.end(); ++it) *it='.';
    }
    
    void append(int i, int j) {
	a_.push_back(i);
	b_.push_back(j);
    }

    void add_basepairA(int i, int j) {
	strA_[i]='(';
	strA_[j]=')';
    }
    void add_basepairB(int i, int j) {
	strB_[i]='(';
	strB_[j]=')';
    }

    void write(std::ostream &out, int width) const;
    void write_clustal(std::ostream &out, int width, int score=INT_MIN) const;
   
    void write_pp(std::ostream &out, Bps &bpsA, Bps &bpsB, int width) const;
};

#endif
