#include <string>
#include <iostream>

#include "sequence.hh"
#include <assert.h>

using namespace std;

double AliColumn::prob_equals(const AliColumn &c) const { // return probability that this equals c 
  int sum=0;
  
  for (int i=0; i<size(); i++) {
    for (int j=0; j<c.size(); j++) {
      sum += ((*this)[i]==c[j])?1:0;
    }
  }
  return sum / (double) (size() * c.size());
}


void Sequence::init_buffer(const string &name) {
  rows_=1;
  len_=0;
  resize(0);
  names_.resize(0);
  names_.push_back(name);
}

void Sequence::init_buffer(const Sequence &seq) {
  names_ = seq.names_;
  rows_ = seq.rows_;
  len_=0;
  resize(len_);
}
  
void Sequence::append_row(string name, string seqstr) {
  names_.push_back(name);
    
  if (rows_==0) {
    len_=seqstr.length();
    resize(len_);
  } else {
    assert(len_ == seqstr.length());
  }
  
  for (int i=1; i<=len_; i++) {
    (*this)[i].push_back(seqstr[i-1]);
  }
  
  rows_++;
}


void Sequence::operator += (Sequence s) {
    for (int i=0; i<s.size(); i++)
	push_back(s[i]);
}

void Sequence::operator += (AliColumn c) {
    push_back(c);
}

void Sequence::operator += (char c) {
    int old_size=size();
    resize(old_size+1);
    for (int i=0; i<rows_; i++) {
	(*this)[old_size+1].push_back(c);
    }
}


void Sequence::write(ostream &out, int start, int end) const {
    // write from position start to position end to out
    // prefix output by names
    
    for (int i=0; i<rows_; i++) {
	int ow=out.width(18);
	out << left << names_[i]<<" ";
	out.width(ow);
	for (int j=start; j<=end; j++) {
	    out << (*this)[j][i];
	}
	out <<endl;
    }
}

void Sequence::write(ostream &out) const {
    write(out,1,len_);
}
